package WebAdmin;

# Services for admin server<->web server communication

use ZeusOS;

# ----------------------------------------------------------
# Usage: host port command errormsg
#
# Sends message to webserver (must be done from webadmin machine so
# communication key can be found).
#
# Returns result string, or undef, in which case errormsg (arg4) is
# filled in with the reason.

sub command {
    my( $host, $port, $command ) = @_;  # arg 4 is error string

    my( $commkey, $buf, $response, $hostip );
    my( $commfile ) = "$ENV{ZEUSHOME}/webadmin/conf/commkey";

    $commkey = "";
    
    # Resolve hostname to ip address
    if( ! ($hostip = gethostbyname( $host )) )  {
	$_[3] = "Unable to resolve $host";
	return;
    }
    
    # If there is a communication key in $ZEUSHOME/webadmin/conf/commkey
    # open and read that
    if( -f $commfile ) {
	local $/;
	local *F;

	if( !open( F, $commfile )) {
	    $_[3] = "Unable to open communication key file: $commfile: $!";
	    return;
	}
	$commkey = scalar <F> || "";
	close F;
	$commkey =~ tr/\t\r\n//d;
    }
    
    # Now open socket connection
    local *S;
    if( !socket( S, $ZeusOS::AF_INET, 
		$ZeusOS::SOCK_STREAM, $ZeusOS::IPPROTO_TCP )) {
	$_[3] = "socket() failed: $!";
	return;
    }

    # Now do network communication
    eval {
	# Set a local timeout, and attempt the following code
	local $SIG{ALRM} = sub { die "operation timeout\n" };

        if( $_[4] ) { alarm( $_[4] ) } else { alarm( 60 ) };
	
	connect( S, ZeusOS::pack_sockaddr_in( $port, $hostip )) or
	    die "connect() failed: $!\n";
	
	$command = $commkey."\t".$command;
	syswrite( S, $command, length( $command )) or 
	    die "syswrite() failed: $!\n";
	
	while( sysread( S, $buf, 1024 )) { $response .= $buf; }
	
	alarm( 0 );
    };

    if( $@ ) {
	# the eval failed in some way, set the response code to undef
	# and the error variable appropriately
	$response = undef;
	$_[3] = $@;
    }
    
    close( S );
    return $response;
}


# ----------------------------------------------------------------------------
# Usage: simple HTTP client with configurable timeouts, so it could feasibly
# be used from CGIs and such.
#
# Params
# $0 - Hostname (can include port)
# $1 - Filename
# $2 - Local filename to save output as
# $3 - Connect / data timeout (seconds) (optional)
# $4... - extra headers (optional)
#
# Returns ( errorcode, error message );
sub HttpGet {

   my( $hostname, $filename, $outfile, $timeout ) = @_;
   splice( @_, 0, 4 );
   $timeout ||= 60;

   my $command = "GET $filename HTTP/1.0\r\nHost: $hostname\r\n";
   while( my $header = shift( @_ )) {
      $command .= $header . "\r\n";
   }
   $command .= "\r\n";
   my $port = 80;

   if( $hostname =~ /:/ ) {
      # Hostname contains port
      ( $hostname, $port ) = split( /:/, $hostname );
   }

   alarm( $timeout );
   local $SIG{ALRM} = sub { return ( 500, "Timeout whilst downloading" ) };

   # Resolve hostname to ip address
   my $hostip = gethostbyname( $hostname );
   return( 500, "Unable to resolve hostname $hostname" )
     unless( defined( $hostip ));

   # Now open socket connection
   local *S;
   if( !socket( S, $ZeusOS::AF_INET, 
		$ZeusOS::SOCK_STREAM, $ZeusOS::IPPROTO_TCP )) {
      alarm( 0 );
      return( 500, "socket() failed: $!" );
   }

   if( !connect( S, ZeusOS::pack_sockaddr_in( $port, $hostip ))) {
      close( S );
      alarm( 0 );
      return( 500, "connect() failed: $!" );
   }

   # Write request out to the server
   if( !syswrite( S, $command, length( $command ))) {
      close( S );
      alarm( 0 );
      return( 500, "syswrite() failed: $!" );
   }

   # Read response line, get status and code
   my $status = <S>;
   chomp( $status );
   my( $errorcode, $errormessage ) = $status =~ /\s(\d+)\s(.*)$/;
   if( $errorcode != 200 ) {
      close( S );
      alarm( 0 );
      return( $errorcode, $errormessage );
   }

   # Read headers and crud back
   my $content_length = undef;
   while( <S> ) {
      chomp;
      s/\r//;
      last if( /^$/ );
      my( $key, $value ) = /^(.*?): (.*)$/;
      $content_length = $value if( $key =~ /content-length/i );
      # Reset timeout
      alarm( $timeout );
   }

   # Prepare output file
   local *OUTFILE;
   if( !open( OUTFILE, "> $outfile" )) {
      close( S );
      alarm( 0 );
      return( 500, "Can't open out file $outfile: $!" );
   }

   # Read body data
   my $size = 0;
   while( read( S, $buf, 4096 )) {
      # Reset timeout
      alarm( $timeout );
      while( length( $buf ) != 0 ) {
         my $written = syswrite( OUTFILE, $buf );
         if( !defined( $written )) {
            close( S );
            close( OUTFILE );
            alarm( 0 );
            unlink( $outfile );
            return( 500, "Error writing output file: $!" );
         }
         $size += $written;
         $buf = substr( $buf, $written );
      }
   }
   close( S );
   close( OUTFILE );
   alarm( 0 );

   # If we got a content length, check the body data wasn't truncated
   if( defined( $content_length )) {
      if( $size != $content_length ) {
         unlink( $outfile );
         return( 500, "Connenction broken by remote server" );
      }
   }

   return( $errorcode, "" );
}


# ----------------------------------------------------------------------------
# Parse the ZIP packages file, to check whats available
# Returns a big hash of information
sub ReadAvailablePackages {

   my $packagelist = "$ENV{ZEUSHOME}/zip/available-packages";
   open( LIST, "< $packagelist" ) or return undef;

   my %packages = ();
   my $current = undef;
   while( <LIST> ) {
      next if( /^\#/ );
      next if( /^\s+/ );
      chomp;
      my( $header, $value ) = /^(.*?):\s*(.*)/;
      next unless( defined( $header ));
      if( $header eq "Package" ) {
         $current = $value;
         $packages{$current} = ();
         next;
      } elsif( $header eq "Description" ) {
         # Description headers are weird, because they are multi-line
         $value = "";
         while( <LIST> ) {
            last unless( /^\s/ );
            $value .= $_;
         }
         $packages{$current}{$header} = $value if( defined( $current ));
         $_ ||= "";
         redo;
      }
      $packages{$current}{$header} = $value if( defined( $current ));
   }
   close( LIST );
   return %packages;

}

# ----------------------------------------------------------
# Returns a list of "host:control port" of the backend machines
sub backends {
   my $filename = "$ENV{ZEUSHOME}/webadmin/conf/hosts";
   my @ret;
   local *F;
   local $/ = "\n";
   if( open( F, $filename )) {
      while( <F> ) {
         chomp;
         push @ret, $_;
      }
      close F;
   }
   return @ret;
}

# ----------------------------------------------------------
# Return the architecture of a backend
#
# $0 - backend worker
#
# Returns ( undef,  $arch ) on success
#         ( $error, undef ) on failure
sub GetArchitecture {

   my $distpackage = "$ENV{ZEUSHOME}/webadmin/bin/distpackage";

   # Sanity checks
   return( "$distpackage binary missing", undef ) if( ! -f $distpackage );
   return( "Cannot run $distpackage: $!", undef ) if( ! -x $distpackage );

   my $backend = shift;
   local *DIST;
   open( DIST, "$distpackage getosinfo $backend |" )
     or return ( "Couldn't run $distpackage for backend $backend: $!", undef );
   my $result = <DIST>;
   chomp $result;
   return( "Error whilst contacting backend web server $backend: $result", undef ) if( $result =~ /^FATAL/ );
   my( $arch ) = $result =~ /$backend:\s+(.*)$/;
   return( "Couldn't read architecture of backend web server $backend, please check that the machine is contactable",undef ) unless( defined( $arch ));
   close( DIST );
   return( undef, $arch );
}

# ------------------------------
# Tests to see if we are a Global Export Edition of a product
# (N.B. This must work for both ZAE and ZWS)
sub isGlobalExportEdition
{
   return -f "$ENV{ZEUSHOME}/webadmin/etc/NO_SSL";
}

# ----------------------------------------------------------
# Determine whether we are running in a clustered environment
#
# To work this requires that then *only* entry in the hosts file for an
# integrated setup is "localhost:XXXX".  If this is not the case (i.e.
# someone has changed from localhost to e.g. 127.0.0.1) then their system
# will be interpreted as a clustered install.
#
# Returns 1 if we are clustered
#         0 if integrated with a web server
sub is_clustered() {

  local *HOSTS;
  $clustered=1;
  open( HOSTS, "$ENV{ZEUSHOME}/webadmin/conf/hosts" ) or return $clustered;
  $_ = <HOSTS>;
  if( /^localhost:.*$/ ) {
    $clustered = 0 if !defined( <HOSTS> );
  }
  close( HOSTS );
  return $clustered;
}

1;

